"""

Test for scenario objects

Meant to be subclassed

"""
from django.contrib.auth.models import User
from django.core import serializers
from django.test import TestCase

from installations import models as installations_models

from mptt.utils import tree_item_iterator

import time

from utils import tests as utils_tests
from utils.trees import models as trees_models
from utils.trees import views as trees_views
from utils.versioning.models import WorkVersion, Version

def print_tree(items):
    res =[]
    prefix = ''
    for element,structure in tree_item_iterator(items):
        if structure['new_level']:
            prefix = '-%s' % prefix
        suffix = element.gate and '(A)' or ''
        res.append(u'%s%s%s' % (prefix, element, suffix))
        x = len(structure['closed_levels'])
        if x:
            prefix = prefix[:-x]
    return u'|'.join(res)

class ScenarioTest(TestCase):
    fixtures = ['dummyusers', 'installations']
##    sc_model = Scenario
##    pl_model = ProtectionLayer
##    ev_model = Event
##    me_model = Measure
##    plme_model = ProtectionLayer_Measure
##    ap_model = AttentionPoint
##    sl_model = Suggestionlist
##    tree_model = TreeElement
##    namespace = "degradations"

    def setUp(self):
        self.setPermissions()
        defaultuser = User.objects.get(pk=1)
        self.section = installations_models.Section.objects.filter(version__isnull=True)[0]
        self.scope = self.section.get_auto_scope()
        # make sure the scope is published
        v1 = Version(name='v1', published=True)
        v1.save(defaultuser)
        self.scenario = self.sc_model(name='scenario', scope=self.scope)
        self.scenario.save(defaultuser)
        self.measure = self.me_model(name='M1', type='PL')
        self.measure.save(defaultuser)
        self.protectionlayer = self.pl_model(name='PL')
        self.protectionlayer.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.scenario})
        self.plme = self.plme_model(protectionlayer=self.protectionlayer, measure = self.measure)
        self.plme.save(defaultuser)
        self.attentionpoint = self.ap_model(name='Capacity', type='Specification', measure=self.measure)
        self.attentionpoint.save(defaultuser)

        self.suggestionlist = self.sl_model(name='SL')
        self.suggestionlist.save(defaultuser)

        # Set up this tree:
        #    Scenario S1 (--scope 1)
        #        -C1|--C2|---C3|--C4|---PL1|----C5
        self.treescenario = self.sc_model(name='S1', scope=self.scope)
        self.treescenario.save(defaultuser)
        self.eventc1 = self.ev_model(name='C1')
        self.elc1 = self.eventc1.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.treescenario})
        self.eventc2 = self.ev_model(name='C2')
        self.elc2 = self.eventc2.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elc1.pk), treeelementkwargs={'scenario': self.treescenario})
        self.eventc3 = self.ev_model(name='C3')
        self.elc3 = self.eventc3.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elc2.pk), treeelementkwargs={'scenario': self.treescenario})
        self.eventc4 = self.ev_model(name='C4')
        self.elc4 = self.eventc4.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elc1.pk), treeelementkwargs={'scenario': self.treescenario})
        self.protectionlayer = self.pl_model(name='PL1')
        self.elpl1 = self.protectionlayer.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elc4.pk), treeelementkwargs={'scenario': self.treescenario})
        self.eventc5 = self.ev_model(name='C5')
        self.elc5 = self.eventc5.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elpl1.pk), treeelementkwargs={'scenario': self.treescenario})

    def test_model(self):
        pass

    def test_model_delete(self):
        """
        test delete scenario deletes all subobjects
        """
        sc_c = self.sc_model.objects.count()
        tree_c = self.tree_model.objects.count()
        ev_c = self.ev_model.objects.count()
        pl_c = self.pl_model.objects.count()
        me_c = self.me_model.objects.count()
        plme_c = self.plme_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)
        scenario = self.sc_model(name='scenario')
        scenario.scope=self.scope
        scenario.save(defaultuser)
        event = self.ev_model(name='C1')
        event.save(defaultuser, parent=None, treeelementkwargs={'scenario': scenario})
        protectionlayer = self.pl_model(name='PL')
        protectionlayer.save(defaultuser, parent=None, treeelementkwargs={'scenario': scenario})
        plme = self.plme_model(protectionlayer=protectionlayer, measure=self.measure)
        plme.save(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), sc_c+1)
        self.assertEquals(self.tree_model.objects.count(), tree_c+2)
        self.assertEquals(self.ev_model.objects.count(), ev_c+1)
        self.assertEquals(self.pl_model.objects.count(), pl_c+1)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c+1)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

        self.sc_model.objects.get(pk=scenario.pk).delete(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), sc_c)
        self.assertEquals(self.tree_model.objects.count(), tree_c)
        self.assertEquals(self.ev_model.objects.count(), ev_c)
        self.assertEquals(self.pl_model.objects.count(), pl_c)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

    def test_model_copy(self):
        # do not copy measure and ap
        sc_c = self.sc_model.objects.count()
        tree_c = self.tree_model.objects.count()
        ev_c = self.ev_model.objects.count()
        pl_c = self.pl_model.objects.count()
        pl_sl_c = self.pl_model.objects.filter(SL=True).count()
        me_c = self.me_model.objects.count()
        plme_c = self.plme_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)
        scenario = self.sc_model(name='scenario')
        scenario.scope=self.scope
        scenario.save(defaultuser)
        event = self.ev_model(name='C1')
        el = event.save(defaultuser, parent=None, treeelementkwargs={'scenario': scenario})
        protectionlayer = self.pl_model(name='PL')
        el = protectionlayer.save(defaultuser, parent=el, treeelementkwargs={'scenario': scenario})
        event = self.ev_model(name='C2')
        el = event.save(defaultuser, parent=el, treeelementkwargs={'scenario': scenario})
        plme = self.plme_model(protectionlayer=protectionlayer, measure=self.measure)
        plme.save(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), sc_c+1)
        self.assertEquals(self.tree_model.objects.count(), tree_c+3)
        self.assertEquals(self.ev_model.objects.count(), ev_c+2)
        self.assertEquals(self.pl_model.objects.count(), pl_c+1)
        self.assertEquals(self.pl_model.objects.filter(SL=True).count(), pl_sl_c)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c+1)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

        treeobjects = list(self.tree_model.tree.filter(
            scenario=scenario,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-C1|--PL|---C2')

        # copy scenario
        scenario2 = scenario.copy(defaultuser, scope=self.scope, suffix='[]')

        self.assertEquals(self.sc_model.objects.count(), sc_c+2)
        self.assertEquals(self.tree_model.objects.count(), tree_c+6)
        self.assertEquals(self.ev_model.objects.count(), ev_c+4)
        self.assertEquals(self.pl_model.objects.count(), pl_c+2)
        self.assertEquals(self.pl_model.objects.filter(SL=True).count(), pl_sl_c)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c+1)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

        self.assertNotEquals(scenario2.v_id, scenario.v_id)
        self.assertEquals(scenario2.name, scenario.name+' []')

        treeobjects2 = list(self.tree_model.tree.filter(
            scenario=scenario2,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects2), '-C1|--PL|---C2')
        self.assertNotEquals(treeobjects2[0].v_id, treeobjects[0].v_id)

        # clean up
        self.sc_model.objects.get(pk=scenario.pk).delete(defaultuser)
        self.sc_model.objects.get(pk=scenario2.pk).delete(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), sc_c)
        self.assertEquals(self.tree_model.objects.count(), tree_c)
        self.assertEquals(self.ev_model.objects.count(), ev_c)
        self.assertEquals(self.pl_model.objects.count(), pl_c)
        self.assertEquals(self.pl_model.objects.filter(SL=True).count(), pl_sl_c)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

    def test_model_copy_to_sl(self):
        # do not copy measure and ap
        sc_c = self.sc_model.objects.count()
        tree_c = self.tree_model.objects.count()
        ev_c = self.ev_model.objects.count()
        pl_c = self.pl_model.objects.count()
        pl_sl_c = self.pl_model.objects.filter(SL=True).count()
        me_c = self.me_model.objects.count()
        plme_c = self.plme_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)
        scenario = self.sc_model(name='scenario')
        scenario.scope=self.scope
        scenario.save(defaultuser)
        event = self.ev_model(name='C1')
        el = event.save(defaultuser, parent=None, treeelementkwargs={'scenario': scenario})
        protectionlayer = self.pl_model(name='PL')
        el = protectionlayer.save(defaultuser, parent=el, treeelementkwargs={'scenario': scenario})
        event = self.ev_model(name='C2')
        el = event.save(defaultuser, parent=el, treeelementkwargs={'scenario': scenario})
        plme = self.plme_model(protectionlayer=protectionlayer, measure=self.measure)
        plme.save(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), sc_c+1)
        self.assertEquals(self.tree_model.objects.count(), tree_c+3)
        self.assertEquals(self.ev_model.objects.count(), ev_c+2)
        self.assertEquals(self.pl_model.objects.count(), pl_c+1)
        self.assertEquals(self.pl_model.objects.filter(SL=True).count(), pl_sl_c)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c+1)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

        treeobjects = list(self.tree_model.tree.filter(
            scenario=scenario,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-C1|--PL|---C2')

        # copy scenario
        scenario2 = scenario.copy(defaultuser, suggestionlist=self.suggestionlist, suffix='[]')

        self.assertEquals(self.sc_model.objects.count(), sc_c+2)
        self.assertEquals(self.tree_model.objects.count(), tree_c+6)
        self.assertEquals(self.ev_model.objects.count(), ev_c+4)
        self.assertEquals(self.pl_model.objects.count(), pl_c+2)
        self.assertEquals(self.pl_model.objects.filter(SL=True).count(), pl_sl_c + 1)
        self.assertEquals(self.me_model.objects.count(), me_c)
        # no plme
        self.assertEquals(self.plme_model.objects.count(), plme_c+1)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

        self.assertEquals(self.sc_model.objects.filter(suggestionlist=self.suggestionlist).count(), 1)

        self.assertNotEquals(scenario2.v_id, scenario.v_id)
        self.assertEquals(scenario2.name, scenario.name+' []')

        treeobjects2 = list(self.tree_model.tree.filter(
            scenario=scenario2,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects2), '-C1|--PL|---C2')
        self.assertNotEquals(treeobjects2[0].v_id, treeobjects[0].v_id)

        # clean up
        self.sc_model.objects.get(pk=scenario.pk).delete(defaultuser)
        self.sc_model.objects.get(pk=scenario2.pk).delete(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), sc_c)
        self.assertEquals(self.tree_model.objects.count(), tree_c)
        self.assertEquals(self.ev_model.objects.count(), ev_c)
        self.assertEquals(self.pl_model.objects.count(), pl_c)
        self.assertEquals(self.pl_model.objects.filter(SL=True).count(), pl_sl_c)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

    def test_model_isdirty(self):
        defaultuser = User.objects.get(pk=1)

        v1 = Version(name='v1')
        v1.save(defaultuser)

        self.assertEquals(self.sc_model.objects.filter(dirty=True).count(), 0)
        self.assertEquals(self.tree_model.objects.filter(dirty=True).count(), 0)
        self.assertEquals(self.ev_model.objects.filter(dirty=True).count(), 0)
        self.assertEquals(self.pl_model.objects.filter(dirty=True).count(), 0)

        # move a tree item, making the tree dirty
        self.tree_model.objects.get(pk=self.elc5.pk).move_to(defaultuser, self.tree_model.objects.get(pk=self.elc4.pk), 'last-child')

        self.assertEquals(self.sc_model.objects.filter(dirty=True).count(), 0)
        self.assertEquals(self.tree_model.objects.filter(dirty=True).count(), 2)
        self.assertEquals(self.ev_model.objects.filter(dirty=True).count(), 0)
        self.assertEquals(self.pl_model.objects.filter(dirty=True).count(), 0)

        # check is_dirty on the scenario
        self.treescenario = self.sc_model.objects.get(pk=self.treescenario.pk) #reload
        self.assertEquals(self.treescenario.dirty, False)
        self.assertEquals(self.treescenario.is_dirty(), True)

        # restore
        self.tree_model.objects.get(pk=self.elc5.pk).move_to(defaultuser, self.tree_model.objects.get(pk=self.elpl1.pk), 'last-child')

    def test_model_version(self):
        user = User.objects.get(pk=1)

        sc_c = self.sc_model.objects.count()
        tree_c = self.tree_model.objects.count()
        ev_c = self.ev_model.objects.count()
        pl_c = self.pl_model.objects.count()
        me_c = self.me_model.objects.count()
        plme_c = self.plme_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)

        v1 = Version(name='v1')
        v1.save(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), 2 * sc_c)
        self.assertEquals(self.tree_model.objects.count(), 2 * tree_c)
        self.assertEquals(self.ev_model.objects.count(), 2 * ev_c)
        self.assertEquals(self.pl_model.objects.count(), 2 * pl_c)
        self.assertEquals(self.me_model.objects.count(), 2 * me_c)
        self.assertEquals(self.plme_model.objects.count(), 2 * plme_c)
        self.assertEquals(self.ap_model.objects.count(), 2 * ap_c)

        self.assertEquals(self.sc_model.objects.dirty().count(), 0)
        self.assertEquals(self.tree_model.objects.dirty().count(), 0)
        self.assertEquals(self.ev_model.objects.dirty().count(), 0)
        self.assertEquals(self.pl_model.objects.dirty().count(), 0)
        self.assertEquals(self.me_model.objects.dirty().count(), 0)
        self.assertEquals(self.plme_model.objects.dirty().count(), 0)
        self.assertEquals(self.ap_model.objects.dirty().count(), 0)

        # Change something in a protectionlayer and in a suggestionlist
        self.protectionlayer.name = 'PL1a'
        self.protectionlayer.save(defaultuser)
        self.suggestionlist.name = 'SLa'
        self.suggestionlist.save(defaultuser)

        self.assertEquals(self.pl_model.objects.dirty().count(), 1)
        self.assertEquals(self.sl_model.objects.dirty().count(), 1)

        v2 = Version(name='v2')
        v2.save(user)

        self.assertEquals(self.sl_model.objects.dirty().count(), 0)
        self.assertEquals(self.pl_model.objects.dirty().count(), 0)

        # let's check the name of the sl in v1
        sl = self.sl_model.objects.get_versionedobject(v_id=self.suggestionlist.v_id, version=v1)
        self.assertEquals(sl.name, 'SL')

        # let's check the name of the sl in v2
        sl = self.sl_model.objects.get_versionedobject(v_id=self.suggestionlist.v_id, version=v2)
        self.assertEquals(sl.name, 'SLa')

        # Check restoring a version
        sl = self.sl_model.objects.get_versionedobject(v_id=self.suggestionlist.v_id, version=WorkVersion)
        self.assertEquals(sl.name, 'SLa')

        v1.restore(user)

        sl = self.sl_model.objects.get_versionedobject(v_id=self.suggestionlist.v_id, version=WorkVersion)
        self.assertEquals(sl.name, 'SL')

    def test_model_publish(self):
        user = User.objects.get(pk=1)

        sc_c = self.sc_model.objects.count()
        tree_c = self.tree_model.objects.count()
        ev_c = self.ev_model.objects.count()
        pl_c = self.pl_model.objects.count()
        me_c = self.me_model.objects.count()
        plme_c = self.plme_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)

        v1 = Version(name='v1', published=True)
        v1.save(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), 2 * sc_c)
        self.assertEquals(self.tree_model.objects.count(), 2 * tree_c)
        self.assertEquals(self.ev_model.objects.count(), 2 * ev_c)
        self.assertEquals(self.pl_model.objects.count(), 2 * pl_c)
        self.assertEquals(self.me_model.objects.count(), 2 * me_c)
        self.assertEquals(self.plme_model.objects.count(), 2 * plme_c)
        self.assertEquals(self.ap_model.objects.count(), 2 * ap_c)

        self.assertEquals(self.sc_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.tree_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.ev_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.me_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.plme_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.ap_model.objects.filter(published_dirty=True).count(), 0)

        # Change something in a protectionlayer and in a suggestionlist
        pl = self.tree_model.objects.filter(scenario=self.scenario).filter(elementtype=2, version__isnull=True)[0].content_object
        pl.name = 'PL1a'
        pl.save(defaultuser)
        self.suggestionlist.name = 'SLa'
        self.suggestionlist.save(defaultuser)

        self.assertEquals(self.pl_model.objects.dirty().count(), 1)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)
        self.assertEquals(self.sl_model.objects.dirty().count(), 1)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)

        self.scenario = self.sc_model.objects.get(v_id=self.scenario.v_id, version__isnull=True)
        self.assertEquals(self.scenario.may_publish(defaultuser), True)

        v2 = Version(name='v2', published=True)
        v2.save(user, propagate=False)
        self.scenario.publish(defaultuser, v2)

        # suggestion list is still dirty
        self.assertEquals(self.sl_model.objects.dirty().count(), 1)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)

        # the pl is not
        self.assertEquals(self.pl_model.objects.dirty().count(), 0)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 0)

        # let's check the name of the sl in the published v2
        sl = self.sl_model.objects.get_versionedobject(v_id=self.suggestionlist.v_id, version=v2)
        self.assertEquals(sl.name, 'SL')

    def test_model_dump(self):
        dump = self.scenario.dump()
        objects=serializers.deserialize('json', dump)
        self.assertEquals(len(list(objects)), 6) # scenario, treeelement, PL, ME, PLME and AP

    def test_model_publish_scope(self):
        user = User.objects.get(pk=1)

        sc_c = self.sc_model.objects.count()
        tree_c = self.tree_model.objects.count()
        ev_c = self.ev_model.objects.count()
        pl_c = self.pl_model.objects.count()
        me_c = self.me_model.objects.count()
        plme_c = self.plme_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)

        v1 = Version(name='v1', published=True)
        v1.save(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), 2 * sc_c)
        self.assertEquals(self.tree_model.objects.count(), 2 * tree_c)
        self.assertEquals(self.ev_model.objects.count(), 2 * ev_c)
        self.assertEquals(self.pl_model.objects.count(), 2 * pl_c)
        self.assertEquals(self.me_model.objects.count(), 2 * me_c)
        self.assertEquals(self.plme_model.objects.count(), 2 * plme_c)
        self.assertEquals(self.ap_model.objects.count(), 2 * ap_c)

        self.assertEquals(self.sc_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.tree_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.ev_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.me_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.plme_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.ap_model.objects.filter(published_dirty=True).count(), 0)

        # Change something in a protectionlayer and in a suggestionlist
        self.protectionlayer.name = 'PL1a'
        self.protectionlayer.save(defaultuser)
        self.suggestionlist.name = 'SLa'
        self.suggestionlist.save(defaultuser)

        self.assertEquals(self.pl_model.objects.dirty().count(), 1)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)
        self.assertEquals(self.sl_model.objects.dirty().count(), 1)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)

        v2 = Version(name='v2', published=True)
        v2.save(user, propagate=False)
        self.scope.publish(defaultuser, v2)

        # suggestion list is still dirty
        self.assertEquals(self.sl_model.objects.dirty().count(), 1)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)

        # the pl is not
        self.assertEquals(self.pl_model.objects.dirty().count(), 0)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 0)

        # let's check the name of the sl in the published v2
        sl = self.sl_model.objects.get_versionedobject(v_id=self.suggestionlist.v_id, version=v2)
        self.assertEquals(sl.name, 'SL')

    def test_model_dump_scope(self):
        sc_c = self.sc_model.objects.filter(version__isnull=True).count()
        tree_c = self.tree_model.objects.filter(version__isnull=True).count()
        ev_c = self.ev_model.objects.filter(version__isnull=True).count()
        pl_c = self.pl_model.objects.filter(version__isnull=True).count()
        me_c = self.me_model.objects.filter(version__isnull=True).count()
        plme_c = self.plme_model.objects.filter(version__isnull=True).count()
        ap_c = self.ap_model.objects.filter(version__isnull=True).count()
        dump = self.scope.dump()
        objects=list(serializers.deserialize('json', dump))
        self.assertEquals(len(objects), sc_c + tree_c + ev_c + pl_c + me_c + plme_c + ap_c + 1)

    def test_model_publish_section(self):
        user = User.objects.get(pk=1)

        sc_c = self.sc_model.objects.count()
        tree_c = self.tree_model.objects.count()
        ev_c = self.ev_model.objects.count()
        pl_c = self.pl_model.objects.count()
        me_c = self.me_model.objects.count()
        plme_c = self.plme_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)

        v1 = Version(name='v1', published=True)
        v1.save(defaultuser)

        self.assertEquals(self.sc_model.objects.count(), 2 * sc_c)
        self.assertEquals(self.tree_model.objects.count(), 2 * tree_c)
        self.assertEquals(self.ev_model.objects.count(), 2 * ev_c)
        self.assertEquals(self.pl_model.objects.count(), 2 * pl_c)
        self.assertEquals(self.me_model.objects.count(), 2 * me_c)
        self.assertEquals(self.plme_model.objects.count(), 2 * plme_c)
        self.assertEquals(self.ap_model.objects.count(), 2 * ap_c)

        self.assertEquals(self.sc_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.tree_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.ev_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.me_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.plme_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.ap_model.objects.filter(published_dirty=True).count(), 0)

        # Change something in a protectionlayer and in a suggestionlist
        self.protectionlayer.name = 'PL1a'
        self.protectionlayer.save(defaultuser)
        self.suggestionlist.name = 'SLa'
        self.suggestionlist.save(defaultuser)

        self.assertEquals(self.pl_model.objects.dirty().count(), 1)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)
        self.assertEquals(self.sl_model.objects.dirty().count(), 1)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)

        v2 = Version(name='v2', published=True)
        v2.save(user, propagate=False)
        self.section.publish(defaultuser, v2)

        # suggestion list is still dirty
        self.assertEquals(self.sl_model.objects.dirty().count(), 1)
        self.assertEquals(self.sl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 1)

        # the pl is not
        self.assertEquals(self.pl_model.objects.dirty().count(), 0)
        self.assertEquals(self.pl_model.objects.filter(published_dirty=True, version__isnull=True).count(), 0)

        # let's check the name of the sl in the published v2
        sl = self.sl_model.objects.get_versionedobject(v_id=self.suggestionlist.v_id, version=v2)
        self.assertEquals(sl.name, 'SL')

    def test_model_dump_section(self):
        sc_c = self.sc_model.objects.filter(version__isnull=True).count()
        tree_c = self.tree_model.objects.filter(version__isnull=True).count()
        ev_c = self.ev_model.objects.filter(version__isnull=True).count()
        pl_c = self.pl_model.objects.filter(version__isnull=True).count()
        me_c = self.me_model.objects.filter(version__isnull=True).count()
        plme_c = self.plme_model.objects.filter(version__isnull=True).count()
        ap_c = self.ap_model.objects.filter(version__isnull=True).count()
        dump = self.section.dump()
        objects = list(serializers.deserialize('json', dump))
        # including: section, section_scope, scope, section_substance and section_reaction
        self.assertEquals(len(objects), sc_c + tree_c + ev_c + pl_c + me_c + plme_c + ap_c + 5)

    def test_model_report(self):
        user = User.objects.get(pk=1)
        self.assertTrue('\n'.join(self.scenario.get_report_body(user, 'en')).find('Cause tree'))
        self.assertTrue('\n'.join(self.scenario.get_report_body(user, 'nl')).find('Oorzakenboom'))

    def test_treeelement_delete(self):
        # test deleting treeelement keeps children intact

        defaultuser = User.objects.get(pk=1)
        # Set up this tree:
        #    Scenario S1 (--scope 1)
        #        -C1|--C2|---C3|---C4

        treescenario = self.sc_model(name='S1', scope=self.scope)
        treescenario.save(defaultuser)

        eventc1 = self.ev_model(name='C1')
        elc1 = eventc1.save(defaultuser, parent=None, treeelementkwargs={'scenario': treescenario})

        eventc2 = self.ev_model(name='C2')
        elc2 = eventc2.save(defaultuser, parent=self.tree_model.objects.get(pk=elc1.pk), treeelementkwargs={'scenario': treescenario})

        eventc3 = self.ev_model(name='C3')
        elc3 = eventc3.save(defaultuser, parent=self.tree_model.objects.get(pk=elc2.pk), treeelementkwargs={'scenario': treescenario})

        eventc4 = self.ev_model(name='C4')
        elc4 = eventc4.save(defaultuser, parent=self.tree_model.objects.get(pk=elc2.pk), treeelementkwargs={'scenario': treescenario})

        treeobjects = list(self.tree_model.tree.filter(
            scenario=treescenario,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|---C4')

        eventc2.delete(defaultuser)

        treeobjects = list(self.tree_model.tree.filter(
            scenario=treescenario,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-C1|--C3|--C4')

    def test_scenario_views(self):

        base_url = '/analyses/%s/scenarios/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class scenarios(utils_tests.tablelist_TestDefinition):
            description = 'list of scenarios'
            url = base_url
            users = [1,2,]
            rowcount = 2

        class scenarios_report_select(utils_tests.TestDefinition):
            description = 'report of list of scenarios: select options'
            url = base_url + '?submit_pdf=submit_pdf&filter-name=x'
            users = [1,2,]

        class scenarios_report(utils_tests.postonly_TestDefinition):
            description = 'pdf of list of scenarios'
            url = base_url
            users = [1,2,]
            data = {
                "filter": "submit_pdf=submit_pdf&filter-name=x",
                "submit_pdf": "Generate pdf",
            }

        class scenario_scenario(utils_tests.TestDefinition):
            description = 'display scenario tab scenario'
            url = base_url + '%s/scenario/' % self.scenario.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/scenario/edit/' % self.scenario.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.scenario.v_id, 'users': [1,]},
                {'url': base_url + '%s/move/' % self.scenario.v_id, 'users': [1,]},
                {'url': base_url + '%s/copy/' % self.scenario.v_id, 'users': [1,]},
                {'url': base_url + '%s/movetosl/' % self.scenario.v_id, 'users': []},
                {'url': base_url + '%s/copytosl/' % self.scenario.v_id, 'users': [1,]},
            ]

        class scenario_causetree(utils_tests.TestDefinition):
            description = 'display scenario tab causetree'
            url = base_url + '%s/causetree/' % self.scenario.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/causetree/edit/' % self.scenario.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.scenario.v_id, 'users': []},
            ]

        class scenario_evaluation(utils_tests.TestDefinition):
            description = 'display scenario tab evaluation'
            url = base_url + '%s/evaluation/' % self.scenario.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/evaluation/edit/' % self.scenario.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.scenario.v_id, 'users': []},
            ]

        class scenario_compare_scenario(utils_tests.TestDefinition):
            description = 'compare scenario tab scenario'
            url = base_url + '%s/compare/0/scenario/' % self.scenario.v_id
            users = [1,2,]

        class scenario_compare_causetree(utils_tests.TestDefinition):
            description = 'compare scenario tab causetree'
            url = base_url + '%s/compare/0/causetree/' % self.scenario.v_id
            users = [1,2,]

        class scenario_report(utils_tests.edit_TestDefinition):
            description = 'pdf report for a scenario'
            url = base_url + '%s/pdf/' % self.scenario.v_id
            users = [1,2,]
            data = {
                "submit_pdf": "Generate pdf",
                "causes": True,
                "protectionlayers": True,
                "measures": True,
                "attentionpoints": True,
                "lopapath_details": True,
            }

        class scenario_add(utils_tests.add_TestDefinition):
            description = 'add scenario'
            url = '/analyses/%s/scopes/1/scenarios/add/' % self.namespace
            users = [1,]
            data = {
                "name": "bogus",
                "type": "scentype",
                "description": "",
                "consequence": "wall thinning",
                "management-referer": "whatever"
            }

            def pretest(self, testcase, allowed):
                newvid = self.test_class.sc_model.objects.next_v_id()
                self.redirect_url = base_url + '%s/causetree/' % newvid

            def extra_tests(self, testcase, response, allowed):
                if allowed:
                    scenario = self.test_class.sc_model.objects.order_by('-id')[0]
                    scope = installations_models.Scope.objects.get(pk=1)
                    testcase.assertEquals(scenario.scope, scope)
                    testcase.assertEquals(len(scenario.get_causetreeelements()), 1)

            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.sc_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class scenario_edit(utils_tests.edit_TestDefinition):
            description = 'edit scenario'
            url = base_url + '%s/scenario/edit/' % self.scenario.v_id
            users = [1,]
            data = {
                "name": "scenario2",
                "type": "scentype",
                "description": "",
                "management-referer": base_url + '%s/scenario/' % self.scenario.v_id
            }
            redirect_url = base_url + '%s/scenario/' % self.scenario.v_id

        class scenario_causetree_edit(utils_tests.TestDefinition):
# This only tests the get, not the post
            description = 'edit scenario tab causetree'
            url = base_url + '%s/causetree/edit/' % self.scenario.v_id
            users = [1,]
            contains = [
                {'url': base_url + '%s/causetree/' % self.scenario.v_id, 'users': [1,]},
            ]

        class scenario_evaluation_edit(utils_tests.edit_TestDefinition):
            description = 'edit scenario tab evaluation'
            url = base_url + '%s/evaluation/edit/' % self.scenario.v_id
            users = [1,]
            data = {
                "evaluation": "ok",
                "is_ok": 0,
                "management-referer": base_url + '%s/evaluation/' % self.scenario.v_id
            }
            redirect_url = base_url + '%s/evaluation/' % self.scenario.v_id

        class scenario_delete(utils_tests.delete_TestDefinition):
            description = 'delete scenario'
            users = [1,]
            redirect_url = '/installations/sections/1/scenarios/'
            def pretest(self, testcase, allowed):
                newitem = self.test_class.sc_model(scope_id=1, name='to_delete', type='HP')
                newitem.save(defaultuser)
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                if not allowed:
                    self.test_class.sc_model.objects.get(id=self.pk).delete(defaultuser, override=True)

        class scenario_typelookup(utils_tests.TestDefinition):
            description = 'scenario type lookup'
            url = base_url + 'type_lookup/'
            users = [1,2,]
            data = {
                "term": "cen",
            }
            contains = [
                {'url': 'scentype', 'users': [1,2,]},
            ]

        class scenario_namelookup(utils_tests.TestDefinition):
            description = 'scenario name lookup'
            url = base_url + 'name_lookup/'
            users = [1,2,]
            data = {
                "term": "ena",
            }
            contains = [
                {'url': 'scenario', 'users': [1,2,]},
            ]

        class scenario_move(utils_tests.edit_TestDefinition):
            description = 'move a scenario'
            url = base_url + '%s/move/' % self.scenario.v_id
            users = [1,]
            data = {
                "target": 2,
                "management-referer": base_url + '%s/scenario/' % self.scenario.v_id
            }
            redirect_url = base_url + '%s/scenario/' % self.scenario.v_id

        class scenario_copy(utils_tests.edit_TestDefinition):
            description = 'copy a scenario'
            url = base_url + '%s/copy/' % self.scenario.v_id
            users = [1,]
            data = {
                "target": 1,
                "management-referer": base_url + '%s/scenario/' % self.scenario.v_id
            }
            def pretest(self, testcase, allowed):
                newvid = self.test_class.sc_model.objects.next_v_id()
                self.redirect_url = base_url + '%s/scenario/' % newvid
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.sc_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class scenario_publish(utils_tests.add_TestDefinition):
            description = 'publish scenario'
            url = base_url + '%s/publish/' % self.scenario.v_id
            users = [1,]
            data = {
                "name": "public version scenario",
                "management-referer": base_url
            }
            redirect_url = base_url

        tests = [
            scenarios(self),
            scenarios_report_select(self),
            scenarios_report(self),
            scenario_scenario(self),
            scenario_causetree(self),
            scenario_evaluation(self),
            scenario_compare_scenario(self),
            scenario_compare_causetree(self),
            scenario_report(self),
            scenario_add(self),
            scenario_edit(self),
            scenario_causetree_edit(self),
            scenario_evaluation_edit(self),
            scenario_delete(self),
            scenario_typelookup(self),
            scenario_namelookup(self),
            scenario_move(self),
            scenario_copy(self),
            scenario_publish(self),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_versioned_report(self):
        base_url = '/analyses/%s/scenarios/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        v1 = Version(name='v1', published=True)
        v1.save(defaultuser)

        class scenario_versioned_report(utils_tests.edit_TestDefinition):
            description = 'pdf report for a versioned scenario'
            url = base_url + '%s/pdf/' % self.treescenario.v_id
            users = [1,2,]
            data = {
                "submit_pdf": "Generate pdf",
                "causes": True,
                "protectionlayers": True,
                "measures": True,
                "attentionpoints": True,
                "lopapath_details": True,
            }
            def pretest(self, testcase, allowed):
                # activate version
                response = testcase.client.get('/versioning/versions/%s/activate/' % v1.id)
                if allowed:
                    testcase.assertRedirects(response, '/versioning/versions/%s/version/' % v1.id)

            def posttest(self, testcase, allowed):
                if allowed:
                    testcase.assertEqual(testcase.client.session['version_id'], v1.id)
                # deactivate
                response = testcase.client.get('/versioning/versions/deactivate/')

        tests = [
            scenario_versioned_report(self),
        ]
        utils_tests.run_tests(self, users, tests)

    def test_causetree_views(self):
        base_url = '/analyses/%s/scenarios/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class move(utils_tests.edit_TestDefinition):
            description = 'move element'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "move|%s|%s|inside;" % (self.elc3.v_id, self.elc4.v_id),
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|--C4|---PL1|----C5|---C3')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    self.test_class.tree_model.objects.get(v_id=self.test_class.elc3.v_id, version__isnull=True).move_to(
                        defaultuser,
                        self.test_class.tree_model.objects.get(v_id=self.test_class.elc2.v_id, version__isnull=True), 'last-child')

        class togglegate(utils_tests.edit_TestDefinition):
            description = 'toggle gate'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "togglegate|%s;" % self.elc1.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1(A)|--C2|---C3|--C4|---PL1|----C5')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.tree_model.objects.get(v_id=self.test_class.elc1.v_id, version__isnull=True)
                    x.gate = False
                    x.save(defaultuser)

        class delete(utils_tests.edit_TestDefinition):
            description = 'delete element'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "",
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                # create an extra item to delete
                scenario = self.test_class.treescenario
                instance = self.test_class.ev_model(name='to_delete')
                el = instance.save(defaultuser, parent=self.test_class.tree_model.objects.get(pk=self.test_class.elc4.pk),
                    treeelementkwargs={'scenario': scenario})
                self.v_id = el.v_id

                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5|---to_delete')
                self.data['hash'] = trees_views.security_hash(treeobjects)
                self.data['actions'] = "delete|%s" % self.v_id

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5|---to_delete')
            def posttest(self, testcase, allowed):
                # restore situation
                if not allowed:
                    x = self.test_class.tree_model.objects.get(v_id=self.v_id, version__isnull=True)
                    x.delete(defaultuser)

        class addcause(utils_tests.edit_TestDefinition):
            description = 'add cause'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "add|%s|cause|New cause;" % self.elc4.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5|---New cause')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.tree_model.objects.filter(version__isnull=True).order_by('-id')[0]
                    x.delete(defaultuser)

        class addprotectionlayer(utils_tests.edit_TestDefinition):
            description = 'add protection layer'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "add|%s|protectionlayer|New PL;" % self.elc4.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5|---New PL')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.tree_model.objects.filter(version__isnull=True).order_by('-id')[0]
                    x.delete(defaultuser)

        class insertconsequence(utils_tests.edit_TestDefinition):
            description = 'insert consequence'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "insert|%s|cause|New consequence;" % self.elc3.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---New consequence|----C3|--C4|---PL1|----C5')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.ev_model.objects.get(name="New consequence").get_treeelements()[0]
                    y = self.test_class.tree_model.objects.get(v_id=self.test_class.elc3.v_id, version__isnull=True)
                    y.move_to(defaultuser, x, 'right')
                    x = self.test_class.tree_model.objects.get(pk=x.pk)
                    x.move_to(defaultuser, None, 'last-child')
                    x = self.test_class.tree_model.objects.get(pk=x.pk)
                    x.delete(defaultuser)

        class insertprotectionlayer(utils_tests.edit_TestDefinition):
            description = 'insert protection layer'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "insert|%s|protectionlayer|New PL;" % self.elc3.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---New PL|----C3|--C4|---PL1|----C5')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.pl_model.objects.get(name="New PL").get_treeelements()[0]
                    y = self.test_class.tree_model.objects.get(v_id=self.test_class.elc3.v_id, version__isnull=True)
                    y.move_to(defaultuser, x, 'right')
                    x = self.test_class.tree_model.objects.get(pk=x.pk)
                    x.move_to(defaultuser, None, 'last-child')
                    x = self.test_class.tree_model.objects.get(pk=x.pk)
                    x.delete(defaultuser)

        class rename(utils_tests.edit_TestDefinition):
            description = 'rename element'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "rename|%s|X;" % self.elc4.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--X|---PL1|----C5')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.tree_model.objects.get(v_id=self.test_class.elc4.v_id, version__isnull=True).content_object
                    x.name = 'C4'
                    x.save(defaultuser)

        tests = [
            move(self),
            togglegate(self),
            delete(self),
            addcause(self),
            addprotectionlayer(self),
            insertconsequence(self),
            insertprotectionlayer(self),
            rename(self),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_complexcausetree_views(self):
        base_url = '/analyses/%s/scenarios/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        # only test for one user
        users = (1,)

        class complex(utils_tests.edit_TestDefinition):
            description = 'complex causetree editing'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "move|%s|%s|inside;delete|%s;add|%s|protectionlayer|New protection layer;rename|-1|X;add|-1|cause|New cause;rename|-2|Y;togglegate|%s;" % (
                    self.elc3.v_id, self.elc4.v_id, self.elc5.v_id, self.elc3.v_id, self.elc4.v_id),
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|--C4(A)|---PL1|---C3|----X|-----Y')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # No restore
                pass


        tests = [
            complex(self),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_toolongnames(self):
        base_url = '/analyses/%s/scenarios/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        # only test for one user
        users = (1,)

        class addcause(utils_tests.edit_TestDefinition):
            description = 'add cause'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "add|%s|cause|New cause with a name that is going to be a lot longer than should fit into the name field of the cause object;" % self.elc4.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5|---New cause with a name that is going to be a lot longer than should fit into the name field of the ca')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.tree_model.objects.filter(version__isnull=True).order_by('-id')[0]
                    x.delete(defaultuser)

        class addprotectionlayer(utils_tests.edit_TestDefinition):
            description = 'add protection layer'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "add|%s|protectionlayer|New PL with a name that is going to be a lot longer than should fit into the name field of the protectionlayer object;" % self.elc4.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5|---New PL with a name that is going to be a lot longer than should fit into the name field of the prote')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.tree_model.objects.filter(version__isnull=True).order_by('-id')[0]
                    x.delete(defaultuser)

        class rename(utils_tests.edit_TestDefinition):
            description = 'rename element'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "rename|%s|Renaming a cause with a name that is going to be a lot longer than should fit into the name field of the cause object;" % self.elc4.v_id,
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                if allowed:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--Renaming a cause with a name that is going to be a lot longer than should fit into the name field of|---PL1|----C5')
                else:
                    testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # restore situation
                if allowed:
                    x = self.test_class.tree_model.objects.get(v_id=self.test_class.elc4.v_id, version__isnull=True).content_object
                    x.name = 'C4'
                    x.save(defaultuser)

        tests = [
            addcause(self),
            addprotectionlayer(self),
            rename(self),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_errorcausetree_views(self):
        """
        Test that moving an item as a sibling of itself (which errors on the move)
        is ignored.
        """
        base_url = '/analyses/%s/scenarios/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        # only test for one user
        users = (1,)

        class error1(utils_tests.edit_TestDefinition):
            description = 'causetree editing with error'
            url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
            users = [1,]
            data = {
                "actions": "move|%s|%s|after" % (
                    self.elc3.v_id, self.elc3.v_id),
                "hash": "",
                "management-referer": base_url + '%s/causetree/' % self.treescenario.v_id
            }
            redirect_url = base_url + '%s/causetree/' % self.treescenario.v_id
            def pretest(self, testcase, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
                self.data['hash'] = trees_views.security_hash(treeobjects)

            def extra_tests(self, testcase, response, allowed):
                treeobjects = list(self.test_class.tree_model.tree.filter(
                    scenario=self.test_class.treescenario,
                    ).get_versionedobjects(version=WorkVersion).select_related_versioned())
                # no change
                testcase.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')
            def posttest(self, testcase, allowed):
                # No restore
                pass


        tests = [
            error1(self),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_regression_longscenariolist(self):

        base_url = '/analyses/%s/scenarios/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        users = (1,)

        for i in xrange(1000):
            self.section = installations_models.Section(name='Test')
            self.section.save(defaultuser)
            self.scope = self.section.get_auto_scope()
            self.scenario = self.sc_model(name='scenario', scope=self.scope)
            self.scenario.save(defaultuser)

        class scenarios(utils_tests.tablelist_TestDefinition):
            description = 'list of scenarios'
            url = base_url
            users = [1,]

        tests = [
            scenarios(self),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_causetree_versioning(self):
        user = User.objects.get(pk=1)

        defaultuser = User.objects.get(pk=1)

        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')

        v1 = Version(name='v1')
        v1.save(defaultuser)

        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=v1).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')

        # delete the original root
        # add a new base
        c1 = self.ev_model.objects.get_versionedobject(name='C1', version=WorkVersion)
        c1_el = c1.get_treeelements()[0]
        newbase = self.ev_model(name=u'Newbase')
        treeelementnb = newbase.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.treescenario})
        treeelementnb.move_to(defaultuser, c1_el, 'right')
        c1_el = c1.get_treeelements()[0]
        treeelementnb = newbase.get_treeelements()[0]
        c1_el.move_to(defaultuser, treeelementnb, 'last-child')

        c3 = self.ev_model.objects.get_versionedobject(name='C3', version=WorkVersion)
        c3.delete(defaultuser)

        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-Newbase|--C1|---C2|---C4|----PL1|-----C5')

        v2 = Version(name='v2')
        v2.save(defaultuser)

        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-Newbase|--C1|---C2|---C4|----PL1|-----C5')

        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=v1).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')

        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=v2).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-Newbase|--C1|---C2|---C4|----PL1|-----C5')

        v1.restore(defaultuser)

        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')

        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=v1).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-C1|--C2|---C3|--C4|---PL1|----C5')

        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=v2).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), '-Newbase|--C1|---C2|---C4|----PL1|-----C5')


class ScenarioSeleniumTest(utils_tests.SeleniumTestCase):
    fixtures = ['dummyusers', 'installations']
##    sc_model = Scenario
##    pl_model = ProtectionLayer
##    ev_model = Event
##    me_model = Measure
##    plme_model = ProtectionLayer_Measure
##    ap_model = AttentionPoint
##    sl_model = Suggestionlist
##    tree_model = TreeElement
##    namespace = "degradations"

    def setUp(self):
        self.setPermissions()
        defaultuser = User.objects.get(pk=1)
        self.section = installations_models.Section.objects.filter(version__isnull=True)[0]
        self.scope = self.section.get_auto_scope()

        # Set up this tree:
        #    Scenario S1 (--scope 1)
        #        -C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5
        self.treescenario = self.sc_model(name='S1', scope=self.scope)
        self.treescenario.save(defaultuser)

        self.eventc1 = self.ev_model(name='C1')
        self.elc1 = self.eventc1.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.treescenario})

        self.protectionlayerpl1 = self.pl_model(name='PL1')
        self.elpl1 = self.protectionlayerpl1.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elc1.pk), treeelementkwargs={'scenario': self.treescenario})

        self.protectionlayerpl2 = self.pl_model(name='PL2')
        self.elpl2 = self.protectionlayerpl2.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elpl1.pk), treeelementkwargs={'scenario': self.treescenario})

        self.eventc2 = self.ev_model(name='C2')
        self.elc2 = self.eventc2.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elpl2.pk), treeelementkwargs={'scenario': self.treescenario})

        self.eventc3 = self.ev_model(name='C3')
        self.elc3 = self.eventc3.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elc2.pk), treeelementkwargs={'scenario': self.treescenario})

        self.protectionlayerpl3 = self.pl_model(name='PL3')
        self.elpl3 = self.protectionlayerpl3.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elc3.pk), treeelementkwargs={'scenario': self.treescenario})

        self.eventc4 = self.ev_model(name='C4')
        self.elc4 = self.eventc4.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elpl2.pk), treeelementkwargs={'scenario': self.treescenario})

        self.protectionlayerpl4 = self.pl_model(name='PL4')
        self.elpl4 = self.protectionlayerpl4.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elc4.pk), treeelementkwargs={'scenario': self.treescenario})

        self.eventc5 = self.ev_model(name='C5')
        self.elc5 = self.eventc5.save(defaultuser, parent=self.tree_model.objects.get(pk=self.elpl4.pk), treeelementkwargs={'scenario': self.treescenario})

        self.login('u1', 'test')

    def checktree(self, value):
        treeobjects = list(self.tree_model.tree.filter(
            scenario=self.treescenario,
            ).get_versionedobjects(version=WorkVersion).select_related_versioned())
        self.assertEquals(print_tree(treeobjects), value)

    def test_dblclick_rename(self):
        defaultuser = User.objects.get(pk=1)

        base_url = '/analyses/%s/scenarios/' % self.namespace
        basetree = '-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5'

        # Rename top item

        self.checktree(basetree)

        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        actions = self.selenium.find_element_by_id('id_actions')
        item = self.selenium.find_element_by_id('%s' % self.elc1.v_id).find_element_by_xpath('./a')

        self.double_click(item)

        input = self.selenium.find_element_by_id('%s' % self.elc1.v_id).find_element_by_xpath('./input')
        input.send_keys(u'C1b')
        self.enter_key(input)

        input = self.selenium.find_element_by_id('%s' % self.elc2.v_id).find_element_by_xpath('./a')
        input.click()

        item = self.selenium.find_element_by_id('%s' % self.elc1.v_id).find_element_by_xpath('./a')
        self.assertTrue('C1b' in item.text)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'rename|%s|C1b' % self.elc1.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree.replace('C1', 'C1b'))

        self.eventc1.name = 'C1'
        self.eventc1.save(defaultuser)

        self.checktree(basetree)

        # Rename other item

        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        actions = self.selenium.find_element_by_id('id_actions')
        item = self.selenium.find_element_by_id('%s' % self.elc4.v_id).find_element_by_xpath('./a')

        self.double_click(item)

        input = self.selenium.find_element_by_id('%s' % self.elc4.v_id).find_element_by_xpath('./input')
        input.send_keys(u'C4b')
        self.enter_key(input)

        item = self.selenium.find_element_by_id('%s' % self.elc4.v_id).find_element_by_xpath('./a')
        self.assertTrue('C4b' in item.text)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'rename|%s|C4b' % self.elc4.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree.replace('C4', 'C4b'))

        self.eventc4.name = 'C4'
        self.eventc4.save(defaultuser)

        self.checktree(basetree)

    def xtest_dnd(self):
# not testing dnd now because of missing the target
# solution ?
        defaultuser = User.objects.get(pk=1)

        base_url = '/analyses/%s/scenarios/' % self.namespace
        basetree = '-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5'

        self.checktree(basetree)

        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item1 = self.selenium.find_element_by_id('%s' % self.elc3.v_id).find_element_by_xpath('./a')
        item2 = self.selenium.find_element_by_id('%s' % self.elc4.v_id).find_element_by_xpath('./a')

        self.drag_and_drop(item1, item2)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'move|%s|%s|last' % (self.elc3.v_id, self.elc4.v_id)
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree('-C1|--PL1|---PL2|----C2|----C4|-----PL4|------C5|-----C3|------PL3')

        # now back
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item1 = self.selenium.find_element_by_id('%s' % self.elc3.v_id).find_element_by_xpath('./a')
        item2 = self.selenium.find_element_by_id('%s' % self.elc2.v_id).find_element_by_xpath('./a')

        self.drag_and_drop(item1, item2)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'move|%s|%s|last' % (self.elc3.v_id, self.elc2.v_id)
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree)

    def test_contextmenu(self):
        defaultuser = User.objects.get(pk=1)

        base_url = '/analyses/%s/scenarios/' % self.namespace
        basetree = '-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5'

        self.checktree(basetree)

        # contents

        labels = [
            'Hernoem',
            'Add',
            'Gevolg invoegen',
            'Oorzaak toevoegen',
            'Beveiligingslaag invoegen',
            'Beveiligingslaag toevoegen',
            'Move',
            'Op',
            'Neer',
            'Verwijder',
            'Oorzaaktype wisselen',
            'Wissel poort',
        ]
        checks = {
            'topevent': [True, True, True, True, False, True, False, False, False, True, False, False],
            'event': [True, True, True, True, True, True, False, False, False, True, True, False],
            'pl': [True, True, True, True, True, True, True, True, True, True, False, False],
            'plhigh': [True, True, True, True, True, True, True, False, True, True, False, False],
            'plbottom': [True, True, True, True, True, True, True, True, False, True, False, False],
            'plabovegate': [True, True, True, True, True, True, True, True, False, True, False, False],
            'gate': [False, False, False, False, False, False, False, False, False, False, False, True],
        }

        objs = [
            ('id', '%s' % self.elc1.v_id, 'topevent'),
            ('id', '%s' % self.elc2.v_id, 'event'),
            ('id', '%s' % self.elpl4.v_id, 'pl'),
            ('id', '%s' % self.elpl1.v_id, 'plhigh'),
            ('id', '%s' % self.elpl3.v_id, 'plbottom'),
            ('id', '%s' % self.elpl2.v_id, 'plabovegate'),
            ('xpath', '//li[@id="%s" and @rel="orgate"]' % self.elpl2.v_id, 'gate'),
        ]

        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        for obj in objs:
            if obj[0] == 'id':
                item = self.selenium.find_element_by_id(obj[1]).find_element_by_xpath('./a')
            else:
                item = self.selenium.find_element_by_xpath(obj[1]).find_element_by_xpath('./a')
            self.context_click(item)
            contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')
            for index in xrange(len(labels)):
                self.assertEqual(labels[index] in contextmenu.get_attribute('innerHTML'), checks.get(obj[2])[index])
            item.click()

    def test_contextmenu_rename(self):
        defaultuser = User.objects.get(pk=1)

        base_url = '/analyses/%s/scenarios/' % self.namespace
        basetree = '-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5'

        self.checktree(basetree)

        # Rename an event
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elc2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')
        button = contextmenu.find_element_by_xpath('//a[@rel="rename"]')
        button.click()
        input = self.selenium.find_element_by_id('%s' % self.elc2.v_id).find_element_by_xpath('./input')
        input.send_keys(u'C2b')
        self.enter_key(input)

        item = self.selenium.find_element_by_id('%s' % self.elc2.v_id).find_element_by_xpath('./a')
        self.assertTrue('C2b' in item.text)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'rename|%s|C2b' % self.elc2.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree.replace('C2', 'C2b'))

        self.eventc2.name = 'C2'
        self.eventc2.save(defaultuser)

        self.checktree(basetree)

        # Rename a pl
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')
        button = contextmenu.find_element_by_xpath('//a[@rel="rename"]')
        button.click()
        input = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./input')
        input.send_keys(u'PL2b')
        self.enter_key(input)

        item = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./a')
        self.assertTrue('PL2b' in item.text)
        actions = self.selenium.find_element_by_id('id_actions')
        action = 'rename|%s|PL2b' % self.elpl2.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree.replace('PL2', 'PL2b'))

        self.protectionlayerpl2.name = 'PL2'
        self.protectionlayerpl2.save(defaultuser)

        self.checktree(basetree)

    def test_contextmenu_add(self):
        defaultuser = User.objects.get(pk=1)

        base_url = '/analyses/%s/scenarios/' % self.namespace
        basetree = '-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5'

        self.checktree(basetree)

        # add gevolg
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.click(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_consequence"]'))
        self.click_in_context()
        time.sleep(0.1)

        input = self.selenium.find_element_by_css_selector('input.jstree-rename-input')
        input.send_keys(u'Newevent')
        self.enter_key(input)

        item = self.selenium.find_element_by_id('-1').find_element_by_xpath('./a')
        self.assertTrue('Newevent' in item.text)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'insert|%s|cause|Newevent' % self.elpl2.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree('-C1|--PL1|---Newevent|----PL2|-----C2|------C3|-------PL3|-----C4|------PL4|-------C5')

        newevent = self.ev_model.objects.get(name='Newevent')
        newevent.delete(defaultuser)

        self.checktree(basetree)

        # insert pl
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.click(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_consequence"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="insert_pl"]'))
        self.click_in_context()
        time.sleep(0.1)

        input = self.selenium.find_element_by_css_selector('input.jstree-rename-input')
        input.send_keys(u'Newpl')
        self.enter_key(input)

        item = self.selenium.find_element_by_id('-1').find_element_by_xpath('./a')
        self.assertTrue('Newpl' in item.text)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'insert|%s|protectionlayer|Newpl' % self.elpl2.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree('-C1|--PL1|---Newpl|----PL2|-----C2|------C3|-------PL3|-----C4|------PL4|-------C5')

        newpl = self.pl_model.objects.get(name='Newpl')
        newpl.delete(defaultuser)

        self.checktree(basetree)

        # add oorzaak
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.click(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_consequence"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_cause"]'))
        self.click_in_context()
        time.sleep(0.1)

        input = self.selenium.find_element_by_css_selector('input.jstree-rename-input')
        input.send_keys(u'Newevent')
        self.enter_key(input)

        item = self.selenium.find_element_by_id('-1').find_element_by_xpath('./a')
        self.assertTrue('Newevent' in item.text)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'add|%s|cause|Newevent' % self.elpl2.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree('-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5|----Newevent')

        newevent = self.ev_model.objects.get(name='Newevent')
        newevent.delete(defaultuser)

        self.checktree(basetree)

        # add pl
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.click(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_consequence"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_pl"]'))
        self.click_in_context()
        time.sleep(0.1)

        input = self.selenium.find_element_by_css_selector('input.jstree-rename-input')
        input.send_keys(u'Newpl')
        self.enter_key(input)

        item = self.selenium.find_element_by_id('-1').find_element_by_xpath('./a')
        self.assertTrue('Newpl' in item.text)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'add|%s|protectionlayer|Newpl' % self.elpl2.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree('-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5|----Newpl')

        newpl = self.pl_model.objects.get(name='Newpl')
        newpl.delete(defaultuser)

        self.checktree(basetree)

    def test_contextmenu_move(self):
        defaultuser = User.objects.get(pk=1)

        base_url = '/analyses/%s/scenarios/' % self.namespace
        basetree = '-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5'

        self.checktree(basetree)

        # move up
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="move"]'))
        self.click(contextmenu.find_element_by_xpath('//a[@rel="move"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="pl_up"]'))
        self.click_in_context()
        time.sleep(0.1)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'move|%(c4)s|%(pl2)s|after;move|%(c2)s|%(pl2)s|after;move|%(pl2)s|%(pl1)s|after;move|%(pl1)s|%(pl2)s|last' % {
                'pl1': self.elpl1.v_id,
                'pl2': self.elpl2.v_id,
                'c2': self.elc2.v_id,
                'c4': self.elc4.v_id,
            }
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree('-C1|--PL2|---PL1|----C2|-----C3|------PL3|----C4|-----PL4|------C5')

        # and back down
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="move"]'))
        self.click(contextmenu.find_element_by_xpath('//a[@rel="move"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="pl_down"]'))
        self.click_in_context()
        time.sleep(0.1)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'move|%(c4)s|%(pl1)s|after;move|%(c2)s|%(pl1)s|after;move|%(pl1)s|%(pl2)s|after;move|%(pl2)s|%(pl1)s|last' % {
                'pl1': self.elpl1.v_id,
                'pl2': self.elpl2.v_id,
                'c2': self.elc2.v_id,
                'c4': self.elc4.v_id,
            }
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree)

    def test_contextmenu_delete(self):
        defaultuser = User.objects.get(pk=1)

        base_url = '/analyses/%s/scenarios/' % self.namespace
        basetree = '-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5'

        self.checktree(basetree)

        # test delete, add something first
        # insert pl
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elpl2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.click(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_consequence"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="insert_pl"]'))
        self.click_in_context()
        time.sleep(0.1)

        input = self.selenium.find_element_by_css_selector('input.jstree-rename-input')
        input.send_keys(u'Newpl')
        self.enter_key(input)

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree('-C1|--PL1|---Newpl|----PL2|-----C2|------C3|-------PL3|-----C4|------PL4|-------C5')

        newpl = self.pl_model.objects.get(name='Newpl')
        newel = newpl.get_treeelements()[0]

        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % newel.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="delete_element"]'))
        self.click_in_context()
        time.sleep(0.1)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'delete|%s' % newel.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree)

        # delete from gate
        # add first
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % self.elpl1.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.click(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_consequence"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_cause"]'))
        self.click_in_context()
        time.sleep(0.1)

        input = self.selenium.find_element_by_css_selector('input.jstree-rename-input')
        input.send_keys(u'Newevent1')
        self.enter_key(input)

        item = self.selenium.find_element_by_id('-1').find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.click(contextmenu.find_element_by_xpath('//a[@rel="add"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_consequence"]'))
        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="add_cause"]'))
        self.click_in_context()
        time.sleep(0.1)

        input = self.selenium.find_element_by_css_selector('input.jstree-rename-input')
        input.send_keys(u'Newevent2')
        self.enter_key(input)

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree('-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5|---Newevent1|----Newevent2')

        newevent1 = self.ev_model.objects.get(name='Newevent1')
        newel1 = newevent1.get_treeelements()[0]
        newevent2 = self.ev_model.objects.get(name='Newevent2')
        newel2 = newevent2.get_treeelements()[0]

        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % newel1.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="delete_element"]'))
        self.click_in_context()
        time.sleep(0.1)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'delete|%s' % newel1.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree('-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5|---Newevent2')

        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        item = self.selenium.find_element_by_id('%s' % newel2.v_id).find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="delete_element"]'))
        self.click_in_context()
        time.sleep(0.1)

        actions = self.selenium.find_element_by_id('id_actions')
        action = 'delete|%s' % newel2.v_id
        self.assertTrue(action in actions.get_attribute('value'))

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree)

    def test_contextmenu_toggle(self):
        defaultuser = User.objects.get(pk=1)

        base_url = '/analyses/%s/scenarios/' % self.namespace
        basetree = '-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5'

        self.checktree(basetree)

        # toggle gate
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        li = self.selenium.find_element_by_xpath('//li[@id="%s" and @rel="orgate"]' % self.elpl2.v_id)
        item = li.find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="toggle_gate"]'))
        self.click_in_context()
        time.sleep(0.1)

        self.assertEqual(li.get_attribute('rel'), 'andgate')

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree.replace('PL2', 'PL2(A)'))

        # and back
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        li = self.selenium.find_element_by_xpath('//li[@id="%s" and @rel="andgate"]' % self.elpl2.v_id)
        item = li.find_element_by_xpath('./a')
        self.context_click(item)
        contextmenu = self.selenium.find_element_by_id('vakata-contextmenu')

        self.move_to_element(contextmenu.find_element_by_xpath('//a[@rel="toggle_gate"]'))
        self.click_in_context()
        time.sleep(0.1)

        self.assertEqual(li.get_attribute('rel'), 'orgate')

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

        self.checktree(basetree)


    def test_actionsform(self):
        defaultuser = User.objects.get(pk=1)

        base_url = '/analyses/%s/scenarios/' % self.namespace
        basetree = '-C1|--PL1|---PL2|----C2|-----C3|------PL3|----C4|-----PL4|------C5'

        self.checktree(basetree)

        # actions not visible
        url = base_url + '%s/causetree/edit/' % self.treescenario.v_id
        self.selenium.get('%s%s' % (self.live_server_url, url))

        actions = self.selenium.find_element_by_id('id_actions')
        self.assertFalse(actions.is_displayed())

        submit = self.selenium.find_element_by_xpath('//input[@type="submit"]')
        submit.click()
        time.sleep(0.3)

