"""
Deviations app

App for deviation analyses

"""
from analyses.analysisbase import models as base_models
from analyses.main import analysesmodules

from django.db import models
from django.db.models import signals as django_signals
from django.utils.functional import curry
from django.utils.translation import ugettext_lazy as _

from hazards import models as hazards_models

from installations import models as installations_models

from utils.exchange import exchange
from utils.forms import create_class_with_meta
from utils import reports as utils_reports
from utils.search.models import searches, Search
from utils.strings import add_suffix_to_string, ugettext_tolanguage as _l
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models
from utils.versioning import versionedmptt

class AppConfig(base_models.AppConfig):
    namespace = "deviations"
    tree_root_elementtype = "loc"
    analysistype =  _('Control of deviation')
    analysistype_short =  _('Control of deviation__')
    scenario_displaypermission = 'deviations.display_deviationscenarios'
    scenario_editpermission = 'deviations.edit_deviationscenarios'
    suggestionlist_displaypermission = 'deviations.display_deviationsuggestionlists'
    suggestionlist_editpermission = 'deviations.edit_deviationsuggestionlists'
    scenarios_url = 'deviations_scenarios'
    suggestionlists_url = 'deviations_suggestionlists'
    addurl = 'deviations_scenario_add'
    consequence_label = _('LOC')

    @classmethod
    def get_models(cls):
        return {
            'suggestionlist': DeviationSuggestionlist,
            'scenario': DeviationScenario,
            'treeelement': CauseTreeElement,
            'event': Cause,
            'protectionlayer': ProtectionLayer,
            'measure': DeviationMeasure,
            'protectionlayer_measure': DeviationProtectionLayer_Measure,
            'attentionpoint': AttentionPoint,
            'apsuggestionlist': DeviationAPSuggestionlist,
        }

class DeviationSuggestionlist(base_models.Suggestionlist):
    class Meta:
        permissions = (
            ("display_deviationsuggestionlists", "May display deviation suggestion lists"),
            ("edit_deviationsuggestionlists", "May edit deviation suggestion lists"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

    def collect_objects_extra(self, version, scenarios, filterkwarg):
        if scenarios:
            # dependencies
            items = list(CauseTreeElement.objects.filter(scenario__in=scenarios
                ).get_versionedobjects(version=version
                ).values_list('v_id', flat=True))
            if items:
                return [list(Dependency.objects.filter(**filterkwarg).filter(element1__in=items).get_versionedobjects(version=version))]
        return [[]]

class DeviationScenario(base_models.Scenario):
    suggestionlist = versioning_models.VersionedForeignKey(DeviationSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="scenarios")
    use_LOPA = models.BooleanField(verbose_name = _('Use LOPA'))
    target_frequency = models.FloatField(verbose_name = _('LOPA target freq.'), blank=True, null=True)
    motivation = models.TextField(verbose_name = _('Motivation'), blank=True, null=True)
    lopa_dirty = models.BooleanField(default=True)

    class Meta(base_models.Scenario.Meta):
        ordering = ('type', 'name',)
        permissions = (
            ("display_deviationscenarios", "May display deviation scenarios"),
            ("edit_deviationscenarios", "May edit deviation scenarios"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

    # urls

    @models.permalink
    def get_add_dependency_url(self):
        return ('deviations_scenario_add_dependency', (self.v_id,))


    # method overrides

    def save(self, user, *args, **kwargs):
        if self.pk:
            old = DeviationScenario.objects.get(pk=self.pk)
            # only when target_frequency changes:
            if old.target_frequency != self.target_frequency:
                self.lopa_dirty = True
            if old.use_LOPA != self.use_LOPA:
                self.lopa_dirty = True
        else:
            self.lopa_dirty = True
        super(DeviationScenario, self).save(user, *args, **kwargs)
    save.alters_data = True

    def prepare_object_for_versioning(self, version):
        self.set_lopa_dirty()
        super(DeviationScenario, self).prepare_object_for_versioning(version)

    def set_lopa_dirty(self, flag=True):
        """
        Set the lopa_dirty flag
        Bypass the save in versionedmodel: this does not need to make this object
        version dirty.
        """
        self.lopa_dirty = flag
        super(versioning_models.VersionedModel, self).save()
    set_lopa_dirty.alters_data = True

    # this should become a descriptor
    def dependencies(self):
        if not hasattr(self, '_dependencies'):
            version = getattr(self, '_version', self.version) or versioning_models.WorkVersion
            treeobjects = CauseTreeElement.objects.filter(scenario=self
                ).get_versionedobjects(version=version).values_list('v_id', flat=True)
            if treeobjects:
                self._dependencies  = Dependency.objects.filter(element1__in=treeobjects).get_versionedobjects(version=version).distinct().order_by('element1', "element2").select_related_versioned()
            else:
                self._dependencies  = Dependency.objects.none()
        # optimisation possibility: prefetch treeleaf elements for treeelements
        return self._dependencies

    # custom methods

    def collect_objects_extra(self, version, filterkwarg):
        # dependencies
        items = list(CauseTreeElement.objects.filter(scenario=self
            ).get_versionedobjects(version=version
            ).values_list('v_id', flat=True))
        if items:
            return [list(Dependency.objects.filter(**filterkwarg).filter(element1__in=items).get_versionedobjects(version=version))]
        return [[]]

    def copy_extra(self, user, newelements):
        deps = list(self.dependencies())
        for item in deps:
            el1 = newelements.get(item.element1_id)
            el2 = newelements.get(item.element2_id)
            if el1 and el2:
                newdep = Dependency(element1=el1, element2=el2)
                newdep.save(user)

    def move(self, user, scope=None, suggestionlist=None):
        """
        Move the scenario and its tree to a scope or suggestionlist
        """
        if scope and scope != self.scope:
            self.scope = scope
            self.suggestionlist = None
            self.save(user)
        if suggestionlist and suggestionlist != self.suggestionlist:
            self.scope = None
            self.suggestionlist = suggestionlist
            self.use_LOPA = False
            self.target_frequency = None
            self.evaluation = u''
            self.lopa_dirty = False
            self.is_ok = False
            self.save(user)

    @classmethod
    def get_report_formfields(cls, user):
        from django import forms
        res = super(DeviationScenario, cls).get_report_formfields(user)
        res.append({'name': 'dependencies',
            'field': forms.BooleanField(required=False, label=_('Dependency details'),),
            'children': []})
        res.append({'name': 'lopapath_details',
            'field': forms.BooleanField(required=False, label=_('Lopa path details'),),
            'children': []})
        return res

    def get_report_body(self, user, language=u'en', choices={}, level=1):
        from analyses.deviations import forms as app_forms
        from analyses.deviations import tables as app_tables
        from analyses.deviations import evaluation
        from utils.trees import views as trees_views

        form = app_forms.DeviationScenarioReportSheet(instance=self, omit_empty=True, report=True)
        context = {'object': self, 'form': form, 'title': u'%s: %s' % (_l('Scenario', language), self), 'level': level}
        version = getattr(self, '_version', self.version) or versioning_models.WorkVersion
        treeobjects = self.appconfig.get_models()['treeelement'].tree.filter(scenario=self
            ).get_versionedobjects(version=version).select_related_versioned()
        tables = []
        dependencies = self.dependencies()
        if dependencies:
            # annotate with number
            i = 1
            for item in dependencies:
                item.number = i
                i += 1
            for item in treeobjects:
                for dep in dependencies:
                    if item == dep.element1 or item == dep.element2:
                        if hasattr(item, 'depnumbers'):
                            item.depnumbers.append(dep.number)
                        else:
                            item.depnumbers = [dep.number]
            if not choices.get('dependencies'):
                context['dependencies'] = dependencies
        context['treeobjects'] = trees_views.annotate_tree(treeobjects)
        errors = evaluation.evaluate_causetree_for_deviationscenario(self)
        context['errors'] = errors
        if not errors:
            queryset = self.deviationlopapath_set.all()
            if queryset:
                context['lopapath_details'] = choices.get('lopapath_details')
                context['lopapaths'] = queryset
        context['tables'] = tables
        html = [utils_reports.make_pdf_body(context, language=language, template='deviations/scenario_report.html')]
        html.extend(versioning_models.VersionedModel.get_report_body(self, user, language, choices, level))
        if dependencies and choices.get('dependencies'):
            dephtml = []
            for item in dependencies:
                dephtml.extend(item.get_report_body(user, language, choices, level+2))
            if dephtml:
                html.append(u'<p class="header%s">%s</p>\n' % (level+1, _l('Dependencies', language)))
                html.extend(dephtml)

        # second part, best after dependencies
        html.append(utils_reports.make_pdf_body(context, language=language, template='deviations/scenario_report_lopa.html'))

        if choices.get('causes'):
            c = list(treeobjects.filter(elementtype = EVENT_ID).values_list('object_id', flat=True))
            if c:
                causes = self.appconfig.get_models()['event'].objects.filter(v_id__in=c).get_versionedobjects(version=version)
                causehtml = []
                for item in causes:
                    causehtml.extend(item.get_report_body(user, language, choices, level+2))
                if causehtml:
                    html.append(u'<p class="header%s">%s</p>\n' % (level+1, _l('Cause details', language)))
                    html.extend(causehtml)
        if choices.get('protectionlayers'):
            c = list(treeobjects.filter(elementtype = PROTECTIONLAYER_ID).values_list('object_id', flat=True))
            if c:
                pls = self.appconfig.get_models()['protectionlayer'].objects.filter(v_id__in=c).get_versionedobjects(version=version)
                plshtml = []
                for item in pls:
                    plshtml.extend(item.get_report_body(user, language, choices, level+2))
                if plshtml:
                    html.append(u'<p class="header%s">%s</p>\n' % (level+1, _l('Protection layer details', language)))
                    html.extend(plshtml)
        return html

    @classmethod
    def get_report_css(cls, language=u'en', choices={}):
        css = super(DeviationScenario, cls).get_report_css(language=language, choices=choices)
        css.append('deviations/scenario_report.css')
        if choices.get('lopapath_details'):
            css.extend(DeviationLopaPath.get_report_css(language, choices))
        if choices.get('dependencies'):
            css.extend(Dependency.get_report_css(language, choices))
        return css

    def get_report(self, user, language=u'en', choices={}):
        body_strings = self.get_report_body(user, language, choices)
        css_files = self.__class__.get_report_css(language, choices)
        return utils_reports.make_pdf(body_strings, css_files, language)

EVENT_ID = base_models.EVENT_ID
PROTECTIONLAYER_ID = base_models.PROTECTIONLAYER_ID

class CauseTreeElement(base_models.TreeElement):
    scenario = versioning_models.VersionedForeignKey(DeviationScenario, verbose_name = _('Scenario'))

    appconfig = AppConfig

    def dependencies(self):
        # this would better be a descriptor
        version = getattr(self, '_version', self.version) or WorkVersion
        pp = list(self.dependencies_forward.all().values_list('element2_id', flat=True))
        pp.extend(list(self.dependencies_reverse.all().values_list('element1_id', flat=True)))
        if pp:
            return CauseTreeElement.objects.filter(v_id__in=pp).get_versionedobjects(version=version)
        return CauseTreeElement.objects.none()

    # method overrides
    def save(self, user, *args, **kwargs):
        self.scenario.set_lopa_dirty()
        super(CauseTreeElement, self).save(user, *args, **kwargs)
    save.alters_data = True

    def delete(self, user, *args, **kwargs):
        self.scenario.set_lopa_dirty()
        # remove dependencies first
        for item in self.dependencies_forward.all():
            item.delete(user, *args, **kwargs)
        for item in self.dependencies_reverse.all():
            item.delete(user, *args, **kwargs)
        super(CauseTreeElement, self).delete(user, *args, **kwargs)
    delete.alters_data = True

    def make_scenario_lopa_dirty(func):
        """
        Decorator to make the scenario lopa_dirty when changing the tree.
        """
        def _wrapper(self, user, *args, **kwargs):
            self.scenario.set_lopa_dirty()
            return func(self, user, *args, **kwargs)
        return _wrapper

    insert_at = make_scenario_lopa_dirty(trees_models.TreeElement.insert_at)
    move_to = make_scenario_lopa_dirty(trees_models.TreeElement.move_to)

class Cause(base_models.Event):
    condition = models.BooleanField(verbose_name=_('Condition'))
    frequency = models.FloatField(verbose_name = _('Frequency'), blank=True, null=True)
    probability = models.FloatField(verbose_name = _('Probability'), blank=True, null=True)

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

    class Meta:
        verbose_name = _('Cause')
        verbose_name_plural = _('Causes')

    # method overrides
    def save(self, user, *args, **kwargs):
        sss = self.get_treeelements()
        for ss in sss:
            ss.scenario.set_lopa_dirty()
        return super(Cause, self).save(user, *args, **kwargs)
    save.alters_data = True

    # from Treeleaf:
    def tree_display(self):
        if self.condition:
            if self.probability:
                return u'%s (prob.: %s %%)' % (self.name, self.probability * 100)
        else:
            if self.frequency:
                return u'%s (freq.: %s/j)' % (self.name, self.frequency)
        return self.name

    def get_elementtype_for_tree(self):
        return self.condition and 'condition' or 'event'

class DeviationMeasure(base_models.Measure):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ProtectionLayer(base_models.ProtectionLayer):
    measures = versioning_models.VersionedManyToManyField(DeviationMeasure,
        verbose_name = _('Measures'),
        through="DeviationProtectionLayer_Measure",
        blank=True, null=True,
        related_name='protectionlayers')

    reltypeChoices = (
        (1, _('PFD')),
        (2, _('Failure frequency')),
        (3, _('Availability')),
    )

    reltype = models.SmallIntegerField(choices=reltypeChoices, default=1)
    rel = models.FloatField(verbose_name = _('PFD / FF'), blank=True, null=True)

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

    # method overrides
    def save(self, user, *args, **kwargs):
        sss = self.get_treeelements()
        for ss in sss:
            ss.scenario.set_lopa_dirty()
        return super(ProtectionLayer, self).save(user, *args, **kwargs)
    save.alters_data = True

    # from Treeleaf:
    def tree_display(self):
        if self.rel:
            if self.reltype == 2:
                return u'%s (FF: %s/j)' % (self.name, self.rel)
            if self.reltype == 1:
                return u'%s (PFD: %s)' % (self.name, self.rel)
            if self.reltype == 3:
                return u'%s (avail: %s %%)' % (self.name, self.rel * 100)
        return self.name

    def get_report_reportsheet(self):
        from analyses.deviations import forms as app_forms
        return app_forms.ProtectionLayerSheet

class DeviationProtectionLayer_Measure(base_models.ProtectionLayer_Measure):
    protectionlayer = versioning_models.VersionedForeignKey(ProtectionLayer)
    measure = versioning_models.VersionedForeignKey(DeviationMeasure)

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class DeviationAPSuggestionlist(base_models.APSuggestionlist):
    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class AttentionPoint(base_models.AttentionPoint):
##
##    protectionlayer = versioning_models.VersionedForeignKey(ProtectionLayer,
##        verbose_name = _('Protection layer'), blank=True, null=True)
##
    measure = versioning_models.VersionedForeignKey(DeviationMeasure, verbose_name = _('Measure'),
        blank=True, null=True,
        related_name="attentionpoints")
    suggestionlist = versioning_models.VersionedForeignKey(DeviationAPSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="attentionpoints")

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission


class Dependency(versioning_models.VersionedModel):
    element1 = versioning_models.VersionedForeignKey(CauseTreeElement, verbose_name = _('Protection layer'), related_name="dependencies_forward")
    element2 = versioning_models.VersionedForeignKey(CauseTreeElement, verbose_name = _('dependent of'), related_name="dependencies_reverse")
    description = models.TextField(verbose_name = _('Description'), blank=True, null=True)

    appconfig = AppConfig

    class Meta:
        verbose_name = _('Dependency')
        verbose_name_plural = _('Dependencies')

    def __unicode__(self):
        return u'Dependency of %s on %s' % (self.element1, self.element2)


    # method overrides

    def save(self, user, *args, **kwargs):
        """
        Checking both elements are from the same tree
        """
        if self.element1.tree_id != self.element2.tree_id:
            raise Exception('Unable to save dependency. Elements do not belong to the same tree')
        self.element1.scenario.set_lopa_dirty()
        super(Dependency, self).save(user, *args, **kwargs)
    save.alters_data = True

    def delete(self, user, *args, **kwargs):
        self.element1.scenario.set_lopa_dirty()
        super(Dependency, self).delete(user, *args, **kwargs)
    delete.alters_data = True

    # urls

    @models.permalink
    def get_absolute_url(self, tab=None):
        return ('deviations_dependency', (self.v_id,))

    @models.permalink
    def get_edit_url(self, tab=None):
        return ('deviations_dependency_edit', (self.v_id,))

    @models.permalink
    def get_delete_url(self):
        return ('deviations_dependency_delete', (self.v_id,))

    @models.permalink
    def get_compare_url(self, version):
        return ('deviations_dependency_compare', (self.v_id, version.id))


    # permissions

    @classmethod
    def may_add(cls, user, parent):
        return parent.may_change(user)

    def may_change(self, user):
        if self.version:
            return False
        return self.element1.scenario.may_change(user)

    def may_delete(self, user):
        if self.version:
            return False
        return self.element1.scenario.may_change(user)

    @classmethod
    def get_report_formfields(cls, user):
        return []

    def get_report_body(self, user, language=u'en', choices={}, level=1, **kwargs):
        from analyses.deviations import forms as app_forms
        subhtml = super(Dependency, self).get_report_body(user, language, choices, level)
        form = create_class_with_meta(app_forms.DependencyReportSheet, model=self.__class__)(instance=self, omit_empty=True, report=True)
        context = {'object': self, 'form': form,
            'title': u'%s %s' % (_l('Dependency', language), self.number),
            'level': level}
        html = [utils_reports.make_pdf_body(context, language=language, template='deviations/dependency_report.html')]
        html.extend(subhtml)
        return html

    @classmethod
    def get_report_css(cls, language=u'en', choices={}, **kwargs):
        css = super(Dependency, cls).get_report_css(language=language, choices=choices)
        css.append('deviations/dependency_report.css')
        return css

    def get_report(self, user, language=u'en', choices={}, **kwargs):
        body_strings = self.get_report_body(user, language, choices)
        css_files = self.__class__.get_report_css(language, choices)
        return utils_reports.make_pdf(body_strings, css_files, language)

def make_version(sender, instance, objects, appconfig=None, **kwargs):
    """
    deviations make_version, should be curried with appconfig object
    adds versioning dependencies
    """
    if not objects:
        # only version everything when objects are not specifically included
        if instance.published:
            filterkwarg={'published_dirty': True, 'version__isnull': True}
        else:
            filterkwarg={'dirty': True, 'version__isnull': True}
        for modelname in ['suggestionlist', 'scenario', 'event', 'protectionlayer',
            'measure', 'protectionlayer_measure', 'apsuggestionlist', 'attentionpoint']:
            for item in appconfig.get_model(modelname).objects.filter(**filterkwarg):
                item.make_version(instance)
        for item in appconfig.get_model('treeelement').tree.root_nodes().filter(**filterkwarg):
            item.make_version(instance)
        for item in Dependency.objects.filter(**filterkwarg):
            item.make_version(instance)

def restore_version(sender, instance, objects, appconfig=None, **kwargs):
    """
    deviations restore_version, should be curried with appconfig object
    adds versioning dependencies
    """
    if not objects:
        for modelname in ['suggestionlist', 'scenario', 'event', 'protectionlayer',
            'measure', 'protectionlayer_measure', 'apsuggestionlist', 'attentionpoint']:
            for item in appconfig.get_model(modelname).objects.get_newerobjects(instance):
                item.restore_version(instance)
        # using the more efficient restore on objects
        appconfig.get_model('treeelement').objects.restore_version(instance)
        for item in Dependency.objects.get_newerobjects(instance):
            item.restore_version(instance)

make_deviations_version = curry(make_version, appconfig=AppConfig)
versioning_models.post_version.connect(make_deviations_version, sender=versioning_models.Version)

restore_deviations_version = curry(restore_version, appconfig=AppConfig)
versioning_models.restore_version.connect(restore_deviations_version, sender=versioning_models.Version)

analysesmodules.register(AppConfig, 10)

class Deviations_SuggestionlistSearch(Search):
    permission = 'deviations.display_deviationsuggestionlists'
    fields = ['name', 'description']
    model = DeviationSuggestionlist
    name = _('Suggestion list')

searches.register(Deviations_SuggestionlistSearch)

class Deviations_ScenarioSearch(Search):
    permission = 'deviations.display_deviationscenarios'
    fields = ['name', 'description', 'evaluation']
    model = DeviationScenario
    name = _('Scenario')

searches.register(Deviations_ScenarioSearch)

class Deviations_CauseSearch(Search):
    permission = 'deviations.display_deviationscenarios'
    fields = ['name', 'description']
    model = Cause
    name = _('Cause')

searches.register(Deviations_CauseSearch)

class Deviations_ProtectionLayerSearch(Search):
    permission = 'deviations.display_deviationscenarios'
    fields = ['name', 'description']
    model = ProtectionLayer
    name = _('Protection layer')

searches.register(Deviations_ProtectionLayerSearch)

class Deviations_MeasureSearch(Search):
    permission = 'deviations.display_deviationscenarios'
    fields = ['name', 'tag', 'description']
    model = DeviationMeasure
    name = _('Measure')

searches.register(Deviations_MeasureSearch)

class Deviations_APSuggestionlistSearch(Search):
    permission = 'deviations.display_deviationscenarios'
    fields = ['name', 'description']
    model = DeviationAPSuggestionlist
    name = _('Attention point suggestion list')

searches.register(Deviations_APSuggestionlistSearch)

class Deviations_AttentionPointSearch(Search):
    permission = 'deviations.display_deviationscenarios'
    fields = ['name', 'description']
    model = AttentionPoint
    name = _('Attention point')

searches.register(Deviations_AttentionPointSearch)




# models to cache lopa calculation results

class DeviationLopaPath(models.Model):
    scenario = models.ForeignKey(DeviationScenario, verbose_name = _('Scenario'))
    number = models.IntegerField()
    initiating = models.CharField(verbose_name=_('Initiating event'), max_length=100)
    final = models.CharField(verbose_name=_('Final event'), max_length=100)
    frequency = models.FloatField(verbose_name = _('Calculated frequency'), blank=True, null=True)
    target = models.FloatField(verbose_name = _('Target frequency'), blank=True, null=True)
    high = models.BooleanField()

    class Meta:
        verbose_name = _('LOPA path')
        verbose_name_plural = _('LOPA paths')
        ordering = ('number',)

    def __unicode__(self):
        return u'%s - %s' % (self.initiating, self.final)

    # urls

    @models.permalink
    def get_absolute_url(self):
        return ('deviations_lopapath', (self.id,))

    @models.permalink
    def get_report_url(self):
        return ('deviations_lopapath_report', (self.id,))

    @classmethod
    def get_listreport_formfields(cls, user):
        from django import forms
        res = [{'name': 'lopapathdetails',
            'field': forms.BooleanField(required=False, label=_('Lopa path details'),),
            'children': cls.get_report_formfields(user)}]
        return res

    @classmethod
    def get_listreport_css(cls, language=u'en', choices={}, **kwargs):
        return ['deviations/lopapaths_report.css']

    @classmethod
    def get_report_formfields(cls, user):
        return []

    def get_report_body(self, user, language=u'en', choices={}, level=1):
        context = {'object': self, 'level': level}
        html = utils_reports.make_pdf_body(context, language=language, template = 'deviations/scenario_path_report.html')
        return [html,]

    @classmethod
    def get_report_css(cls, language=u'en', choices={}):
        return ['deviations/scenario_path_report.css']

    def get_report(self, user, language=u'en', choices={}):
        body_strings = self.get_report_body(user, language, choices)
        css_files = self.__class__.get_report_css(language, choices)
        return utils_reports.make_pdf(body_strings, css_files, language)

class DeviationLopaPathElement(models.Model):
    path = models.ForeignKey(DeviationLopaPath, verbose_name = _('Path'))
    number = models.IntegerField()
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    prob = models.FloatField(verbose_name = _('Prob'), blank=True, null=True)
    freq = models.FloatField(verbose_name = _('Freq'), blank=True, null=True)
    disabled = models.BooleanField()
    elementtype = models.CharField(verbose_name=_('Elementtype'), max_length=100)

    class Meta:
        ordering = ('number',)
