"""
Helper functions for the import and export of data

"""
import copy
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.core import serializers
from django.core.serializers.json import Serializer as jsonSerializer
from django.db import models
from StringIO import StringIO
from utils.versioning.models import VersionedModel, VersionedForeignKey, VersionedGenericForeignKey
from utils.versioning.versionedmptt import VersionedMpttModel
from utils.trees.models import TreeLeaf

class vfkSerializer(jsonSerializer):
    def serialize(self, queryset, **options):
        """
        Serialize a queryset.
        """
        self.options = options

        self.stream = options.pop("stream", StringIO())
        self.selected_fields = options.pop("fields", None)
        self.use_natural_keys = options.pop("use_natural_keys", False)

        self.start_serialization()
        for obj in queryset:
            self.start_object(obj)
            for field in obj._meta.local_fields:
                if field.serialize:
                    if field.rel is None:
                        if self.selected_fields is None or field.attname in self.selected_fields:
                            if isinstance(field, VersionedForeignKey):
                                self.handle_vfk_field(obj, field)
                            else:
                                self.handle_field(obj, field)
                    else:
                        if self.selected_fields is None or field.attname[:-3] in self.selected_fields:
                            self.handle_fk_field(obj, field)
            for field in obj._meta.many_to_many:
                if field.serialize:
                    if self.selected_fields is None or field.attname in self.selected_fields:
                        self.handle_m2m_field(obj, field)
            self.end_object(obj)
        self.end_serialization()
        return self.getvalue()

    def handle_vfk_field(self, obj, field):
        related = getattr(obj, field.name)
        if related is not None:
            if self.use_natural_keys and hasattr(related, 'natural_key'):
                related = related.natural_key()
            else:
                # Related to remote object via v_id
                related = related.v_id
        self._current[field.name] = related

def dump(objects):
    ser = vfkSerializer()
    ser.serialize(objects, indent=4, use_natural_keys=True)
    return ser.getvalue()
##    return serializers.serialize('json', objects, indent=4, use_natural_keys=True)

def load(user, data, pkmap={}, v_idmap ={}, set_dirty=True):
    """ Loads a json string as new objects into the database
    Keeps a mapping of foreignkeys to redirect them to the right objects.
    You can preload this mapping with the pkmap parameter.
    """
    load_objects(user, objects=serializers.deserialize('json', data), pkmap=pkmap, v_idmap=v_idmap, set_dirty=set_dirty)

def load_objects(user, objects=[], pkmap=None, v_idmap =None, level = 0, set_dirty=True):
    if level > 5:
        raise Exception('Loading data impossible: recursion depth exceeded')
    else:
        if pkmap:
            pkdict = copy.deepcopy(pkmap)
        else:
            pkdict = {}
        if v_idmap:
            v_iddict = copy.deepcopy(v_idmap)
        else:
            v_iddict = {}
        delayed_objects = []
        for deserialized_object in objects:
            newobject = deserialized_object.object
            delayed = False
            # check fields for FK, VFK, GFK, or GVFK
# this should be once for each model
            excluded_fields = ['version', 'user_created', 'user_changed', 'content_type']
            vfk_fields = [g for g in newobject._meta.fields if isinstance(g, VersionedForeignKey) and g.name not in excluded_fields]
            fk_fields = [g for g in newobject._meta.fields if isinstance(g, models.ForeignKey) and g.name not in excluded_fields]
            vgfk_fields = [g for g in newobject._meta.virtual_fields if isinstance(g, VersionedGenericForeignKey) and g.name not in excluded_fields]
            gfk_fields = [g for g in newobject._meta.virtual_fields if isinstance(g, generic.GenericForeignKey) and g.name not in excluded_fields and g not in vgfk_fields]


            # see if foreign objects already loaded
            fks_to_set = {}
            for f in fk_fields:
                ct = ContentType.objects.get_for_model(f.rel.to).pk
                pk = getattr(newobject, f.get_attname())
                if pk:
                    foreign_object = pkdict.get((ct,pk))
                    if not foreign_object:
##                        print 'delayed fk field', f.name
                        delayed = True
                    else:
                        fks_to_set[f.name] = foreign_object

            for f in vfk_fields:
                ct = ContentType.objects.get_for_model(f.to).pk
                v_id = getattr(newobject, f.get_attname())
                if v_id:
                    if hasattr(v_id, '__iter__'):
                        if hasattr(f.to._default_manager, 'get_by_natural_key'):
                            foreign_object = f.to._default_manager.get_by_natural_key(*v_id)
                    else:
                        foreign_object = v_iddict.get((ct,v_id))
                    if not foreign_object:
##                        print 'delayed vfk_field', f.name
                        delayed = True
                    else:
                        fks_to_set[f.name] = foreign_object

            for f in gfk_fields:
                c = getattr(newobject, f.ct_field)
                if c:
                    ct = c.pk
                pk = getattr(newobject, f.fk_field)
                if ct and pk:
                    foreign_object = pkdict.get((ct,pk))
                    if not foreign_object:
##                        print 'delayed gfk_field', f.name
                        delayed = True
                    else:
                        fks_to_set[f.name] = foreign_object
            for f in vgfk_fields:
                c = getattr(newobject, f.ct_field)
                if c:
                    ct = c.pk
                v_id = getattr(newobject, f.fk_field)
                if ct and v_id:
                    foreign_object = v_iddict.get((ct,v_id))
                    if not foreign_object:
##                        print 'delayed vgfk_field', f.name
                        delayed = True
                    else:
                        fks_to_set[f.name] = foreign_object

            if not delayed:
                for k,v in fks_to_set.items():
                    setattr(newobject, k, v)
                # save object
                loaded_pk = newobject.pk
                newobject.pk = None
                if isinstance(newobject, VersionedModel):
                    loaded_v_id = newobject.v_id
                    newobject.v_id = None
                    newobject.version = None
                    newobject.published = False
                    newobject.pk = None
                    if isinstance(newobject, VersionedMpttModel):
                        if newobject.parent:
                            newobject.tree_id = newobject.parent.tree_id
                        else:
                            newobject.tree_id = newobject.__class__.tree._get_next_tree_id()
                    if isinstance(newobject, TreeLeaf):
                        newobject.save(user, save_treeelement=False, set_dirty=set_dirty)
                    elif hasattr(newobject, 'save_accepts_raw'):
                        newobject.save(user, raw=True, set_dirty=set_dirty)
                    else:
                        newobject.save(user, set_dirty=set_dirty)
                else:
                    newobject.save()
                # keep new id
                ct = ContentType.objects.get_for_model(newobject.__class__).pk
                if isinstance(newobject, VersionedModel):
                    v_iddict[(ct, loaded_v_id)] = newobject
                pkdict[(ct, loaded_pk)] = newobject
            else:
                delayed_objects.append(deserialized_object)

        if delayed_objects:
            load_objects(user, objects=delayed_objects, pkmap=pkdict, v_idmap=v_iddict, level=level+1, set_dirty=set_dirty)

