"""
custom fields
"""

import datetime, time

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.markup.templatetags.markup import markdown
from django.db import models
from django import forms
from django.utils.encoding import smart_unicode, force_unicode
from django.utils.html import escape, conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _, ugettext as _g_

from utils import sheets

# Form fields

def friendly_username(user):
    value = user
    if isinstance(user, User):
        if user.last_name:
            if user.first_name:
                value = "%s %s" % (user.last_name, user.first_name)
            else:
                value = user.last_name
        else:
            if user.first_name:
                value = user.first_name
    return value

class UserModelChoiceField(forms.ModelChoiceField):
    """
    ModelChoiceField displaying username as lastname firstname
    """
    def label_from_instance(self, obj):
        from django.contrib.auth.models import User
        if isinstance(obj, User):
            return friendly_username(obj)
        return "--"

class UserModelMultipleChoiceField(forms.ModelMultipleChoiceField):
    """
    ModelMultipleChoiceField displaying username as lastname firstname
    """
    def label_from_instance(self, obj):
        from django.contrib.auth.models import User
        if isinstance(obj, User):
            return friendly_username(obj)
        return "--"

class DateFormField(forms.DateField):
    """
    Specific date formfield
    Allows entering dates without the year part which resolves into the current year
    """
    def __init__(self, input_formats=None, *args, **kwargs):
        super(DateFormField, self).__init__(*args, **kwargs)
        self.input_formats = input_formats or settings.DATE_INPUT_FORMATS

    def clean(self, value):
        super(forms.DateField, self).clean(value)
        if value in forms.fields.EMPTY_VALUES:
            return None
        if isinstance(value, datetime.datetime):
            return value.date()
        if isinstance(value, datetime.date):
            return value
        for format in self.input_formats:
            try:
                y, m, d = time.strptime(value, format)[:3]
                if y == 1900:
                    y = datetime.datetime.now().year
                return datetime.date(y, m, d)
            except ValueError:
                continue
        raise forms.util.ValidationError(self.error_messages['invalid'])

# model fields

MONTH_CHOICES = [('', '------'),]
for i in range(1,13):
    MONTH_CHOICES.append((i, _(datetime.date.strftime(datetime.date(2000,i,1),'%B'))))

class MonthFormField(forms.ChoiceField):
    """
    Form field for selecting a month
    """
    def __init__(self, *args, **kwargs):
        super(MonthFormField, self).__init__(*args, **kwargs)
        self.choices = MONTH_CHOICES

# sheet fields
class UserSheetManyToManyField(sheets.ManyToManyField):
    """
    SheetField displaying username as lastname firstname
    """
    def format_value(self, value):
        value = friendly_username(value)
        return mark_safe(u'%s' % conditional_escape(force_unicode(value)))

class UserSheetForeignKeyField(sheets.ForeignKeyField):
    """
    SheetField displaying username as lastname firstname
    """
    def format_value(self, value):
        value = friendly_username(value)
        return mark_safe(u'%s' % conditional_escape(force_unicode(value)))

class MarkdownField(sheets.SheetField):
    def render(self, sheet, instance):
        if sheet.omit_empty:
            if not self.get_data(instance):
                return u''
        # overridden to take sheet.report into account
        field = self.render_field(instance, sheet.report)
        label = self.render_label(instance)
        return u'<tr>%(label)s%(field)s</tr>' % {'field': field, 'label': label}

    def render_field(self, instance, report=False):
        """
        We break the rendered markdown into different table rows in a report, because
        pisa does not know how to split long table rows.
        """
        field = self.get_field(instance)
        if report:
            field = field.replace(u'</p>\n<p>', u'</p></td></tr>\n<tr><th>&nbsp;</th><td><p>')
            return u'<td>%(field)s</td>' % {'field': field}
        else:
            id = self.get_id(instance)
            return u'<td><div id="%(id)s">%(field)s</div></td>' % {'field': field, 'id': id}

    def format_value(self, value):
        return u'%s' % markdown(force_unicode(value), 'tables')
